/* Skeleton module.
 *
 * Skeleton module (c) 2005 PegSoft
 * Contact us at pegsoft@pegsoft.net
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program (see the file COPYING); if not, write to the
 * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

#include <system/headers.h>

#include <core/version.h>
#include <log/levels.h>
#include <modules/core.h>

#include "decl.h"
#include "module.h"

using namespace std;
using namespace modules;
using namespace skeleton;

/*************************************************************************/

/* This function creates the module object.
 * It is called by the module manager to load the module. 
 *
 * It must be named MODULENAME_get, where MODULENAME is the name of
 * the module with any non alphanumeric character replaced with _.
 * That's a way to avoid name collisions.
 *
 * It must be extern "C" so that name mangling doesn't take place,
 * which is needed in order to resolve the symbol.
 *
 * \param[in] core The core module.
 * \return The newly-created module, or 0 if something went wrong. */

extern "C" Module *skeleton_get(Core &core)
{
	/* Ensure the core version is the same as the version we were
	 * compiled for so that there are no compatibility problems later. */
	if (core.version() != core::core_version)
	{
		core.log(xlog::LogCritical(), "Core module version mismatch");
		return 0;
	}
	
	return (me = new Skeleton(core));
}

/*************************************************************************/

/* This function destroys the module object, and does any remaining
 * cleaning work so that the whole module is in the same state it was
 * before the get function was called. Called by the module manager
 * when the module has been fully unloaded.
 *
 * This IS necessary because when modules are linked statically (i.e.
 * when no dlopen mechanism is available), the global variables are
 * initialized at program start only, there is no external loading.
 *
 * See the *_get function for information about naming scheme and
 * calling convention. 
 *
 * \param[in] core The core module.
 * \param[in] module Pointer to the module to release. */

extern "C" void skeleton_release(Core &core, Module *module)
{
	delete module;
	me = 0;
}

/*************************************************************************/

namespace skeleton
{

/*************************************************************************/

Skeleton *me;

/*************************************************************************/

Skeleton::Skeleton(Core &core) : Module("skeleton", "Skeleton example module"),
		m_core(&core)
{
	
}

/*************************************************************************/

} /* namespace skeleton */

/*************************************************************************/
